/*
 DSPAM
 COPYRIGHT (C) 2003 NETWORK DWEEBS CORPORATION

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _LIBDSPAM_OBJECTS_H
#  define _LIBDSPAM_OBJECTS_H

#include "decode.h"

/* spam totals on disk + memory */
struct _ds_spam_totals
{
  long total_spam;
  long total_innocent;
  long spam_misclassified;
  long innocent_misclassified;
  long spam_corpusfed;
  long innocent_corpusfed;
};

/* spam statistic in memory */
struct _ds_spam_stat
{
  double probability;
  long spam_hits;
  long innocent_hits;
  char disk;      /* does stat exist on disk? used only by some drivers */
};

/* neural network node */
struct _ds_neural_record
{
  uid_t uid;
  long total_correct;
  long total_incorrect;
  long control_correct;
  long control_incorrect;
  char disk;
};

/* spam signature */
struct _ds_spam_signature
{
  void *data;
  long length;
};

/* neural decision */
struct _ds_neural_decision
{
  void *data;
  long length;
};

/* signature token */
struct _ds_signature_token
{
  unsigned long long token;
  unsigned char frequency;
};

typedef struct
{
  char *username;               /* Current Username */
  char *group;                  /* Current Group */
  struct _ds_spam_totals totals;        /* Spam totals */
  struct _ds_spam_signature *signature; /* Input/Output Spam Signature */
  struct _ds_message *message;  /* Message Components */
  time_t _process_start;        /* Process start time */
  float probability;            /* DSPAM Probability Result */
  int result;                   /* DSPAM Boolean Result */
  float confidence;             /* DSPAM Confidence in Result */
  int mode;                     /* DSPAM Processor mode */
  int flags;                    /* DSPAM Processor flags */
  int locked;                   /* Is user locked? */
  void *storage;                /* Storage Driver Structure Pointer */
} DSPAM_CTX;

/* NOTE: Confidence is not always supported, and may be 0 */

/* DSPAM Flags */
#define DSF_CHAINED		0x01    /* Use chained tokens */
#define DSF_SIGNATURE		0x02    /* Signature provided/requested */
#define DSF_INOCULATE		0x04    /* Inoculate user */
#define DSF_COPYBACK		0x08    /* Copy back decoded messages */
#define DSF_IGNOREHEADER	0x10    /* Ignore message headers */
#define DSF_CORPUS		0x20    /* Is corpus mail */
#define DSF_NOISE		0x40	/* Bayesian Noise Reduction */

/* DSF_SIGNATURE: If used in DSM_PROCESS or DSM_ADDSPAM mode, will 
                  generate a signature for reversal.  If used in
                  DSM_FALSEPOSITIVE mode, will assume *message is a
                  signature and process it accordingly */

/* DSPAM Processor modes */
#define DSM_PROCESS             0x00
#define DSM_ADDSPAM             0x01
#define DSM_FALSEPOSITIVE       0x02
#define DSM_TOOLS		0x03
#define DSM_CLASSIFY		0x04

/* DSPAM Results */
#define	DSR_ISSPAM		0x01
#define DSR_ISINNOCENT		0x02

#endif
